@@ -45,9 +45,11 @@ struct _GstA52Dec {
   /* pads */
   GstPad        *sinkpad,
                 *srcpad;
+  GstSegment     segment;
 
   gboolean       dvdmode;
   gboolean       sent_segment;
+  gboolean       discont;
 
   int            bit_rate;
   int            sample_rate;
@@ -63,6 +65,9 @@ struct _GstA52Dec {
 
   GstBuffer     *cache;
   GstClockTime   time;
+
+  /* reverse */
+  GList         *queued;
 };
 
 struct _GstA52DecClass {
@@ -73,6 +78,31 @@ struct _GstA52DecClass {
 
 GType gst_a52dec_get_type (void);
 
+#ifndef A52_MONO
+#define A52_MONO 1
+#endif
+#ifndef A52_STEREO
+#define A52_STEREO 2
+#endif
+#ifndef A52_3F
+#define A52_3F 3
+#endif
+#ifndef A52_2F1R
+#define A52_2F1R 4
+#endif
+#ifndef A52_3F1R
+#define A52_3F1R 5
+#endif
+#ifndef A52_2F2R
+#define A52_2F2R 6
+#endif
+#ifndef A52_3F2R
+#define A52_3F2R 7
+#endif
+#ifndef A52_DOLBY
+#define A52_DOLBY 10
+#endif
+
 G_END_DECLS
 
 #endif /* __GST_A52DEC_H__ */
