@@ -104,7 +104,7 @@ static GstClockTime gst_mpeg_parse_adjust_ts (GstMPEGParse * mpeg_parse,
 
 static GstFlowReturn gst_mpeg_parse_send_buffer (GstMPEGParse * mpeg_parse,
     GstBuffer * buffer, GstClockTime time);
-static GstFlowReturn gst_mpeg_parse_process_event (GstMPEGParse * mpeg_parse,
+static gboolean gst_mpeg_parse_process_event (GstMPEGParse * mpeg_parse,
     GstEvent * event);
 static gboolean gst_mpeg_parse_send_event (GstMPEGParse * mpeg_parse,
     GstEvent * event);
@@ -383,9 +383,16 @@ gst_mpeg_parse_process_event (GstMPEGParse * mpeg_parse, GstEvent * event)
           if (CLASS (mpeg_parse)->send_event) {
             CLASS (mpeg_parse)->send_event (mpeg_parse,
                 gst_event_new_new_segment (FALSE, rate, GST_FORMAT_TIME,
-                    start, -1, time));
+                    start, stop, time));
+            mpeg_parse->pending_newsegment = FALSE;
           }
         }
+      } else if (format != GST_FORMAT_TIME && !update) {
+        GST_DEBUG_OBJECT (mpeg_parse,
+            "Received non-time newsegment from stream");
+        mpeg_parse->do_adjust = TRUE;
+        mpeg_parse->adjust = 0;
+        mpeg_parse->pending_newsegment = TRUE;
       }
       mpeg_parse->packetize->resync = TRUE;
 
@@ -407,6 +414,20 @@ gst_mpeg_parse_process_event (GstMPEGParse * mpeg_parse, GstEvent * event)
       gst_mpeg_packetize_flush_cache (mpeg_parse->packetize);
       break;
     }
+    case GST_EVENT_EOS:{
+      GST_DEBUG_OBJECT (mpeg_parse, "EOS");
+
+      if (CLASS (mpeg_parse)->send_event) {
+        ret = CLASS (mpeg_parse)->send_event (mpeg_parse, event);
+      } else {
+        gst_event_unref (event);
+      }
+      if (!ret) {
+        GST_ELEMENT_ERROR (mpeg_parse, STREAM, DEMUX, (NULL),
+            ("Pushing EOS event didn't work on any of the source pads"));
+      }
+      break;
+    }
     default:
       if (CLASS (mpeg_parse)->send_event) {
         ret = CLASS (mpeg_parse)->send_event (mpeg_parse, event);
@@ -419,12 +440,13 @@ gst_mpeg_parse_process_event (GstMPEGParse * mpeg_parse, GstEvent * event)
   return ret;
 }
 
+/* returns TRUE if pushing the event worked on at least one pad */
 static gboolean
 gst_mpeg_parse_send_event (GstMPEGParse * mpeg_parse, GstEvent * event)
 {
   GstIterator *it;
   gpointer pad;
-  gboolean ret = TRUE;
+  gboolean ret = FALSE;
 
   /* Send event to all source pads of this element. */
   it = gst_element_iterate_src_pads (GST_ELEMENT (mpeg_parse));
@@ -432,7 +454,8 @@ gst_mpeg_parse_send_event (GstMPEGParse * mpeg_parse, GstEvent * event)
     switch (gst_iterator_next (it, &pad)) {
       case GST_ITERATOR_OK:
         gst_event_ref (event);
-        gst_pad_push_event (GST_PAD (pad), event);
+        if (gst_pad_push_event (GST_PAD (pad), event))
+          ret = TRUE;
         gst_object_unref (GST_OBJECT (pad));
         break;
       case GST_ITERATOR_RESYNC:
@@ -472,7 +495,7 @@ gst_mpeg_parse_pad_added (GstElement * element, GstPad * pad)
   event = gst_event_new_new_segment (FALSE,
       mpeg_parse->current_segment.rate,
       GST_FORMAT_TIME, mpeg_parse->current_segment.start,
-      -1, mpeg_parse->current_segment.start);
+      mpeg_parse->current_segment.stop, mpeg_parse->current_segment.start);
 
   gst_pad_push_event (pad, event);
 }
@@ -489,8 +512,8 @@ gst_mpeg_parse_parse_packhead (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
   buf = GST_BUFFER_DATA (buffer);
   buf += 4;
 
-  scr1 = GUINT32_FROM_BE (*(guint32 *) buf);
-  scr2 = GUINT32_FROM_BE (*(guint32 *) (buf + 4));
+  scr1 = GST_READ_UINT32_BE (buf);
+  scr2 = GST_READ_UINT32_BE (buf + 4);
 
   /* Extract the SCR and rate values from the header. */
   if (GST_MPEG_PACKETIZE_IS_MPEG2 (mpeg_parse->packetize)) {
@@ -512,7 +535,7 @@ gst_mpeg_parse_parse_packhead (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
         scr - mpeg_parse->current_scr);
 
     buf += 6;
-    new_rate = (GUINT32_FROM_BE ((*(guint32 *) buf)) & 0xfffffc00) >> 10;
+    new_rate = (GST_READ_UINT32_BE (buf) & 0xfffffc00) >> 10;
   } else {
     scr = ((guint64) scr1 & 0x0e000000) << 5;
     scr |= ((guint64) scr1 & 0x00fffe00) << 6;
@@ -684,8 +707,8 @@ gst_mpeg_parse_event (GstPad * pad, GstEvent * event)
 static GstFlowReturn
 gst_mpeg_parse_chain (GstPad * pad, GstBuffer * buffer)
 {
-  GstMPEGParse *mpeg_parse = GST_MPEG_PARSE (gst_pad_get_parent (pad));
-  GstFlowReturn result = GST_FLOW_ERROR;
+  GstMPEGParse *mpeg_parse = GST_MPEG_PARSE (GST_PAD_PARENT (pad));
+  GstFlowReturn result;
   guint id;
   gboolean mpeg2;
   GstClockTime time;
@@ -704,10 +727,10 @@ gst_mpeg_parse_chain (GstPad * pad, GstBuffer * buffer)
     if (result == GST_FLOW_RESEND) {
       /* there was not enough data in packetizer cache */
       result = GST_FLOW_OK;
-      goto done;
+      break;
     }
     if (result != GST_FLOW_OK)
-      goto done;
+      break;
 
     id = GST_MPEG_PACKETIZE_ID (mpeg_parse->packetize);
     mpeg2 = GST_MPEG_PACKETIZE_IS_MPEG2 (mpeg_parse->packetize);
@@ -749,7 +772,7 @@ gst_mpeg_parse_chain (GstPad * pad, GstBuffer * buffer)
       GST_DEBUG_OBJECT (mpeg_parse, "waiting for SCR");
       gst_buffer_unref (buffer);
       result = GST_FLOW_OK;
-      goto done;
+      break;
     }
 
     /* Update the byte count. */
@@ -768,7 +791,7 @@ gst_mpeg_parse_chain (GstPad * pad, GstBuffer * buffer)
         GST_ELEMENT_ERROR (mpeg_parse, CORE, NEGOTIATION, (NULL), (NULL));
         gst_buffer_unref (buffer);
         result = GST_FLOW_ERROR;
-        goto done;
+        break;
       }
     }
 
@@ -819,13 +842,11 @@ gst_mpeg_parse_chain (GstPad * pad, GstBuffer * buffer)
           ", total since SCR: %" G_GINT64_FORMAT ", br: %" G_GINT64_FORMAT
           ", next SCR: %" G_GINT64_FORMAT, size, bss, br, mpeg_parse->next_scr);
     }
-  } while (!GST_FLOW_IS_FATAL (result));
+  } while (GST_FLOW_IS_SUCCESS (result));
 
-done:
-  gst_object_unref (mpeg_parse);
-
-  if (result == GST_FLOW_NOT_LINKED)
-    result = GST_FLOW_OK;
+  if (result != GST_FLOW_OK) {
+    GST_DEBUG_OBJECT (mpeg_parse, "flow: %s", gst_flow_get_name (result));
+  }
 
   return result;
 }
@@ -988,13 +1009,14 @@ gboolean
 gst_mpeg_parse_handle_src_query (GstPad * pad, GstQuery * query)
 {
   gboolean res = TRUE;
-  GstMPEGParse *mpeg_parse = GST_MPEG_PARSE (GST_PAD_PARENT (pad));
+  GstMPEGParse *mpeg_parse;
   GstFormat src_format = GST_FORMAT_UNDEFINED, format;
-  gint64 src_value = 0, value;
+  gint64 src_value = 0, value = -1;
+
+  mpeg_parse = GST_MPEG_PARSE (gst_pad_get_parent (pad));
 
   switch (GST_QUERY_TYPE (query)) {
-    case GST_QUERY_DURATION:
-    {
+    case GST_QUERY_DURATION:{
       gst_query_parse_duration (query, &format, NULL);
 
       switch (format) {
@@ -1005,7 +1027,7 @@ gst_mpeg_parse_handle_src_query (GstPad * pad, GstQuery * query)
           /* Try asking upstream if it knows the time - a DVD might
              know */
           src_format = GST_FORMAT_TIME;
-          if (gst_pad_query_duration (GST_PAD_PEER (mpeg_parse->sinkpad),
+          if (gst_pad_query_peer_duration (mpeg_parse->sinkpad,
                   &src_format, &src_value)) {
             res = TRUE;
             break;
@@ -1013,7 +1035,7 @@ gst_mpeg_parse_handle_src_query (GstPad * pad, GstQuery * query)
           /* Otherwise fallthrough */
         default:
           src_format = GST_FORMAT_BYTES;
-          if (!gst_pad_query_duration (GST_PAD_PEER (mpeg_parse->sinkpad),
+          if (!gst_pad_query_peer_duration (mpeg_parse->sinkpad,
                   &src_format, &src_value)) {
             res = FALSE;
             goto done;
@@ -1022,17 +1044,15 @@ gst_mpeg_parse_handle_src_query (GstPad * pad, GstQuery * query)
       }
 
       /* Convert the value to the desired format. */
-      if (!gst_mpeg_parse_convert (mpeg_parse, src_format, src_value,
-              &format, &value)) {
-        res = FALSE;
-        goto done;
+      if ((res = gst_mpeg_parse_convert (mpeg_parse, src_format, src_value,
+                  &format, &value)) ||
+          (res = gst_pad_query_peer_duration (mpeg_parse->sinkpad,
+                  &format, &value))) {
+        gst_query_set_duration (query, format, value);
       }
-
-      gst_query_set_duration (query, format, value);
       break;
     }
-    case GST_QUERY_POSITION:
-    {
+    case GST_QUERY_POSITION:{
       gint64 cur;
 
       gst_query_parse_position (query, &format, NULL);
@@ -1055,26 +1075,23 @@ gst_mpeg_parse_handle_src_query (GstPad * pad, GstQuery * query)
       }
 
       /* Convert the value to the desired format. */
-      if (!gst_mpeg_parse_convert (mpeg_parse, src_format, src_value,
-              &format, &value)) {
-        res = FALSE;
-        goto done;
+      if ((res = gst_mpeg_parse_convert (mpeg_parse, src_format, src_value,
+                  &format, &value)) ||
+          (res = gst_pad_query_peer_position (mpeg_parse->sinkpad,
+                  &format, &value))) {
+        gst_query_set_position (query, format, value);
       }
-
-      gst_query_set_position (query, format, value);
       break;
     }
-    case GST_QUERY_CONVERT:
-    {
+    case GST_QUERY_CONVERT:{
       gst_query_parse_convert (query, &src_format, &src_value, &format, NULL);
 
-      if (!gst_mpeg_parse_convert (mpeg_parse, src_format, src_value,
-              &format, &value)) {
-        res = FALSE;
-        goto done;
+      if ((res = gst_mpeg_parse_convert (mpeg_parse, src_format, src_value,
+                  &format, &value)) ||
+          (res = gst_pad_query_peer_convert (mpeg_parse->sinkpad,
+                  src_format, src_value, &format, &value))) {
+        gst_query_set_convert (query, src_format, src_value, format, value);
       }
-
-      gst_query_set_convert (query, src_format, src_value, format, value);
       break;
     }
     default:
@@ -1083,6 +1100,7 @@ gst_mpeg_parse_handle_src_query (GstPad * pad, GstQuery * query)
   }
 
 done:
+  gst_object_unref (mpeg_parse);
   return res;
 }
 
