@@ -1,6 +1,8 @@
-/* gstsynaesthesia.c: implementation of synaesthesia drawing element
+/* GStreamer
  * Copyright (C) <2001> Richard Boulton <richard@tartarus.org>
  *
+ * gstsynaesthesia.c: implementation of synaesthesia drawing element
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -19,18 +21,14 @@
  * SECTION:element-synaesthesia
  * @see_also: goom
  *
- * <refsect2>
- * <para>
  * Synaesthesia is an audio visualisation element. It creates glitter and
  * pulsating fog based on the incomming audio signal.
- * </para>
+ * <refsect2>
  * <title>Example launch line</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch -v audiotestsrc ! audioconvert ! synaesthesia ! ximagesink
  * gst-launch -v audiotestsrc ! audioconvert ! synaesthesia ! ffmpegcolorspace ! xvimagesink
- * </programlisting>
- * </para>
+ * ]|
  * </refsect2>
  */
 
@@ -38,64 +36,7 @@
 #include "config.h"
 #endif
 
-#include <string.h>
-#include <gst/gst.h>
-#include <gst/video/video.h>
-#include <gst/audio/audio.h>
-#include <gst/base/gstadapter.h>
-#include "synaescope.h"
-
-#define SYNAES_SAMPLES 512
-#define SYNAES_WIDTH 320
-#define SYNAES_HEIGHT 200
-
-#define GST_TYPE_SYNAESTHESIA            (gst_synaesthesia_get_type())
-#define GST_SYNAESTHESIA(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_SYNAESTHESIA,GstSynaesthesia))
-#define GST_SYNAESTHESIA_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_SYNAESTHESIA,GstSynaesthesiaClass))
-#define GST_IS_SYNAESTHESIA(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_SYNAESTHESIA))
-#define GST_IS_SYNAESTHESIA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_SYNAESTHESIA))
-
-typedef struct _GstSynaesthesia GstSynaesthesia;
-typedef struct _GstSynaesthesiaClass GstSynaesthesiaClass;
-
-GST_DEBUG_CATEGORY_STATIC (synaesthesia_debug);
-#define GST_CAT_DEFAULT (synaesthesia_debug)
-
-struct _GstSynaesthesia
-{
-  GstElement element;
-
-  /* pads */
-  GstPad *sinkpad, *srcpad;
-  GstAdapter *adapter;
-
-  guint64 next_ts;              /* the timestamp of the next frame */
-  guint64 frame_duration;
-  guint bps;                    /* bytes per sample        */
-  guint spf;                    /* samples per video frame */
-
-  gint16 datain[2][SYNAES_SAMPLES];
-
-  /* video state */
-  gint fps_n, fps_d;
-  gint width;
-  gint height;
-  gint channels;
-
-  /* Audio state */
-  gint sample_rate;
-  gint rate;
-
-  /* Synaesthesia instance */
-  syn_instance *si;
-};
-
-struct _GstSynaesthesiaClass
-{
-  GstElementClass parent_class;
-};
-
-GType gst_synaesthesia_get_type (void);
+#include "gstsynaesthesia.h"
 
 
 /* elementfactory information */
